/* 
    ======================================================================================= 
     "This sample program is provided AS IS and may be used, executed, copied and modified
     without royalty payment by customer

     (a) for its own instruction and study,
     (b) in order to develop applications designed to run with an IBM WebSphere product,
         either for customer's own internal use or for redistribution by customer, as part
         of such an application, in customer's own products."

     Product 5655-FLW,  (C) COPYRIGHT International Business Machines Corp., 2006, 2010
     All Rights Reserved * Licensed Materials - Property of IBM
    ======================================================================================= 
*/
package bpc.samples.command;
import java.util.List;
import java.util.logging.Logger;

import bpc.samples.CustomException;
import bpc.samples.ServiceFactory;
import bpc.samples.handler.CustomHandler;

import com.ibm.bpc.clientcore.ClientException;
import com.ibm.bpc.clientcore.Command;
import com.ibm.bpe.api.ActivityServiceTemplateData;
import com.ibm.bpe.api.BusinessFlowManagerService;
import com.ibm.bpe.api.ClientObjectWrapper;
import com.ibm.bpe.api.PIID;
import com.ibm.bpe.api.ProcessTemplateData;
import commonj.sdo.DataObject;

/** 
 * StartProcessInstance will complete task instances provided in the execute method
 * through the BusinessFlowManagerService.initiate(..) method. Please refer to the 
 * BusinessFlowManagerService API for more details.
 * 
 * <p>Any input message must be provided by the containing object referenced in the
 * context attribute of the command tag, e.g a handler.</p>
 * 
 * <p>Any error during the execution will be wrapped in a CustomException. Add more
 * error handling here if wanted.</p> 

 */
public class StartProcessInstance implements Command {

    static Logger logger = Logger.getLogger("CustomClient") ;
    
    private CustomHandler	handler ;
    
    
    /** Start process instance from the selected process template
     *  
     * @param selectedObjects: the process 
     * @see com.ibm.bpc.clientcore.Command#execute(java.util.List)
     */
    public String execute(List selectedObjects) throws ClientException {
        logger.info( "Executing StartProcessInstance command.") ;
        
		try
		{			
	        BusinessFlowManagerService bfm = ServiceFactory.getInstance().getBusinessFlowManagerService();
	        
	        //Get the process template information
		    ProcessTemplateData template = handler.getProcessTemplate() ;
		    
		    //Get all start activities of the process
		    //Because the OrderManagement process has only one start activity
		    //   the first one is choosen
		    ActivityServiceTemplateData[] startActivities = bfm.getStartActivities(template.getID());
		    
		    //Create the input message
		    ClientObjectWrapper inputMessage = bfm.createMessage(
		    		startActivities[0].getServiceTemplateID(),
		    		startActivities[0].getActivityTemplateID(),
		    		startActivities[0].getInputMessageTypeName() );
		    
		    //Now fill the values into the input message
		    DataObject dataObject = (DataObject)inputMessage.getObject();
		    //create business object "OrderData"
		    DataObject orderData = dataObject.createDataObject("order");
		    
		    if(handler.getSupplierNo() != null)
		    {
		    	orderData.setString("supplierNo", handler.getSupplierNo());
		    }
		    if(handler.getSizeOfOrder() != null)
		    {
		    	orderData.setDouble("sizeOfOrder", Double.parseDouble(handler.getSizeOfOrder()));
		    }
		    if(handler.getOrderText() != null)
		    {
		    	orderData.setString("orderText",handler.getOrderText());
		    }
		    
			// Start the process
		    PIID piid = bfm.initiate(
		    		startActivities[0].getServiceTemplateID(),
		    		startActivities[0].getActivityTemplateID(),
					getProcessName(),
					inputMessage  );
		    
			//set the custom properties at the process instance
			if(handler.getOrderNo() != null) {
				
				logger.info("orderNo: " + handler.getOrderNo());
				
				bfm.setCustomProperty(piid,"orderNo",handler.getOrderNo());
			}						
			
			if(handler.getDepartment() != null)	{
				
				logger.info("department: " + handler.getDepartment());
				
				bfm.setCustomProperty(piid,"department", handler.getDepartment() );
			}
			
			//the process was started successfully - now delete the input values of the custom handler
			handler.resetInputValues();
		}
		catch(Exception e) {
			throw new CustomException( "Process start failed: " + e.getMessage() ) ;
		}		

        return null;
    }

    /* (non-Javadoc)
     * @see com.ibm.bpc.clientcore.Command#isMultiSelectEnabled()
     */
    public boolean isMultiSelectEnabled() {
        return false;
    }

    /* (non-Javadoc)
     * @see com.ibm.bpc.clientcore.Command#isApplicable(java.util.List)
     */
    public boolean[] isApplicable(List itemsOnList) {
        return null;
    }

    /**<p>Sets the context for the command.</p>
     * <p>This command works on an input message for the selected process template.
     * When the command is executed, it will retrieve the message and use it to
     * start a new instance through the BusinessFlowManaher API
     * 
     * @param context: the message for the process start 
     * @see com.ibm.bpc.clientcore.Command#setContext(java.lang.Object)
     */
    public void setContext(Object context) {
       if( context != null && context instanceof CustomHandler ) {
           handler = (CustomHandler) context ;
           logger.info( "Set handler for process start.") ;
       }
    }
    /**
     * Compute the process instance name using the process template name and the order number
     * This name MUST be unique!!! 
     * 
     * @return String representing the process instance name
     */
    private String getProcessName()
    {
	    //Compute a name for the process instance
		//Note: this name must be UNIQUE!
    	if(handler != null)
    	{
    	    StringBuffer piName = new StringBuffer(128);
    	    
		    if(handler.getSupplierNo() != null && handler.getSupplierNo().length() > 0)
		    {
		    	
		    	piName.append("Supplier no: ");
		    	piName.append(handler.getSupplierNo());
		    }
		    else if(handler.getOrderText() != null && handler.getOrderText().length() > 0)
		    {
		    	piName.append("Order text: ");
		    	piName.append(handler.getOrderText());
		    }
		    else
		    {
		    	piName.append(handler.getProcessTemplate().getName());
		    }
		    
            // make the process instance name (probably) unique...
		    if(handler.getOrderNo() != null && handler.getOrderNo().length() > 0)
		    {
		    	piName.append(" - Order no: ");
		    	piName.append(handler.getOrderNo());
		    }
		    else
		    {
		    	piName.append(" - ");
		    	piName.append(System.currentTimeMillis());
		    }
	    	return piName.toString();		    
    	}
    	else
    	{
    		return null; // let the process container take the process instance ID as name
    	}
    }
}
